@@ -47,9 +47,9 @@ BYT;
 """
         expected = [
             {'number': 1, 'start': 1, 'end': 501, 'size': 500,
-             'filesystem': 'ext4', 'flags': 'boot'},
+             'filesystem': 'ext4', 'partition_name': '', 'flags': 'boot'},
             {'number': 2, 'start': 501, 'end': 476940, 'size': 476439,
-             'filesystem': '', 'flags': ''},
+             'filesystem': '', 'partition_name': '', 'flags': ''},
         ]
         execute_mock.return_value = (output, '')
         result = disk_utils.list_partitions('/dev/fake')
@@ -70,6 +70,46 @@ BYT;
         self.assertEqual(1, log_mock.call_count)
 
 
+@mock.patch.object(utils, 'execute', autospec=True)
+class ListPartitionsGPTTestCase(base.IronicLibTestCase):
+
+    def test_correct(self, execute_mock):
+        output = """
+BYT;
+/dev/vda:40960MiB:virtblk:512:512:gpt:Virtio Block Device:;
+2:1.00MiB:2.00MiB:1.00MiB::Bios partition:bios_grub;
+1:4.00MiB:5407MiB:5403MiB:ext4:Root partition:;
+3:5407MiB:5507MiB:100MiB:fat16:Boot partition:boot, esp;
+"""
+        expected = [
+            {'end': 2, 'number': 2, 'start': 1, 'flags': 'bios_grub',
+             'filesystem': '', 'partition_name': 'Bios partition', 'size': 1},
+            {'end': 5407, 'number': 1, 'start': 4, 'flags': '',
+             'filesystem': 'ext4', 'partition_name': 'Root partition',
+             'size': 5403},
+            {'end': 5507, 'number': 3, 'start': 5407,
+             'flags': 'boot, esp', 'filesystem': 'fat16',
+             'partition_name': 'Boot partition', 'size': 100},
+        ]
+        execute_mock.return_value = (output, '')
+        result = disk_utils.list_partitions('/dev/fake')
+        self.assertEqual(expected, result)
+        execute_mock.assert_called_once_with(
+            'parted', '-s', '-m', '/dev/fake', 'unit', 'MiB', 'print',
+            use_standard_locale=True, run_as_root=True)
+
+    @mock.patch.object(disk_utils.LOG, 'warning', autospec=True)
+    def test_incorrect(self, log_mock, execute_mock):
+        output = """
+BYT;
+/dev/vda:40960MiB:virtblk:512:512:gpt:Virtio Block Device:;
+2:XX1.00MiB:---:1.00MiB::primary:bios_grub;
+"""
+        execute_mock.return_value = (output, '')
+        self.assertEqual([], disk_utils.list_partitions('/dev/fake'))
+        self.assertEqual(1, log_mock.call_count)
+
+
 @mock.patch.object(disk_partitioner.DiskPartitioner, 'commit', lambda _: None)
 class WorkOnDiskTestCase(base.IronicLibTestCase):
 
@@ -1246,6 +1286,11 @@ class WholeDiskConfigDriveTestCases(base.IronicLibTestCase):
         mock_execute.assert_has_calls([
             mock.call('sgdisk', '-n', '0:-64MB:0', self.dev,
                       run_as_root=True),
+            mock.call('sync'),
+            mock.call('udevadm', 'settle'),
+            mock.call('partprobe', self.dev, attempts=10, run_as_root=True),
+            mock.call('sgdisk', '-v', self.dev, run_as_root=True),
+
             mock.call('udevadm', 'settle'),
             mock.call('test', '-e', expected_part, attempts=15,
                       check_exit_code=[0], delay_on_retry=True)
@@ -1347,6 +1392,10 @@ class WholeDiskConfigDriveTestCases(base.IronicLibTestCase):
                                     '-0', run_as_root=True)
         mock_execute.assert_has_calls([
             parted_call,
+            mock.call('sync'),
+            mock.call('udevadm', 'settle'),
+            mock.call('partprobe', self.dev, attempts=10, run_as_root=True),
+            mock.call('sgdisk', '-v', self.dev, run_as_root=True),
             mock.call('udevadm', 'settle'),
             mock.call('test', '-e', expected_part, attempts=15,
                       check_exit_code=[0], delay_on_retry=True)
@@ -1443,10 +1492,17 @@ class WholeDiskConfigDriveTestCases(base.IronicLibTestCase):
                                self.node_uuid, self.dev, config_url)
 
         mock_get_configdrive.assert_called_with(config_url, self.node_uuid)
-        mock_execute.assert_called_with('parted', '-a', 'optimal', '-s', '--',
-                                        self.dev, 'mkpart', 'primary',
-                                        'fat32', '-64MiB', '-0',
-                                        run_as_root=True)
+        mock_execute.assert_has_calls([
+            mock.call('parted', '-a', 'optimal', '-s', '--',
+                      self.dev, 'mkpart', 'primary',
+                      'fat32', '-64MiB', '-0',
+                      run_as_root=True),
+            mock.call('sync'),
+            mock.call('udevadm', 'settle'),
+            mock.call('partprobe', self.dev, attempts=10, run_as_root=True),
+            mock.call('sgdisk', '-v', self.dev, run_as_root=True),
+        ])
+
         self.assertEqual(2, mock_list_partitions.call_count)
         mock_fix_gpt_partition.assert_called_with(self.dev, self.node_uuid)
         mock_is_disk_gpt.assert_called_with(self.dev, self.node_uuid)
